﻿Clear-Host;

$_tenantId = '5bfd7eb0-164b-4ab1-99c2-2d6f1b77bd58';        # Tenant ID/Directory ID

$_appName = 'K2 Integration';                              # Your Custom Application Name

$_url = 'https://k2.denallix.com';                        # Your K2 Site URL

function Load-AzModule {

    $_module = Get-Module -Name AzureAD -ErrorAction Ignore;

    if (!$_module) {

        Write-Host "Installing NuGet Package Provider..." -ForegroundColor Cyan;

        Install-PackageProvider -Name NuGet -MinimumVersion 2.8.5.201 -Force -Confirm:$false | Out-Null;

        Write-Host "Installing AzureAD module..." -ForegroundColor Cyan;

        Install-Module AzureAD -SkipPublisherCheck -Force -Confirm:$false -WarningAction SilentlyContinue | Out-Null;

        Write-Host "Importing AzureAD module..." -ForegroundColor Cyan;

        Import-Module AzureAD -Force -WarningAction SilentlyContinue | Out-Null;
    }
}

function Add-ClientSecret {

    param
    (
        $Application
    );

    $_a = "abcdefghijklmnopqrstuvwxyz";

    $_b = "ABCDEFGHIJKLMNOPQRSTUVWXYZ";
    
    $_c = '12345656789';
    
    $_d = '!#$%*+-=?@^_';
    
    $_chars = [System.Text.StringBuilder]::new();
    
    $null = $_chars.Append($_a).Append($_b).Append($_c).Append($_d);
    
    $_chars = $_chars.ToString();
    
    $_secret = [System.Text.StringBuilder]::new();
    
    for ($_i = 0; $_i -lt 16; $_i += 1) {

        $null = $_secret.Append($_chars[(Get-Random -Minimum 0 -Maximum ($_chars.Length))]);
    }
    
    Write-Host "Adding Client Secret..." -ForegroundColor Cyan;

    $_appCredential = New-AzureADApplicationPasswordCredential -ObjectId $Application.ObjectId -CustomKeyIdentifier "Client Secret" -EndDate (Get-Date).AddMonths(6).ToString() -Value "$_secret";
    
    return $_secret;
}

function Add-HighTrustCertificate {

    param
    (
        $Application
    );

    Write-Host 'Exporting K2 OAuth High Trust Certificate' -ForegroundColor Cyan;

    $_certificate = Get-ChildItem cert:\LocalMachine\MY | Where-Object { $_.Subject -eq 'CN=K2 OAuth High Trust'};

    Export-Certificate -Cert $_certificate -FilePath ".\K2 OAuth High Trust.cer" | Out-Null;

    $_oAuthCertificate = New-Object System.Security.Cryptography.X509Certificates.X509Certificate;

    $_oAuthCertificate.Import(".\K2 OAuth High Trust.cer");

    $_rawCertificate = $_oAuthCertificate.GetRawCertData();

    $_value = [System.Convert]::ToBase64String($_rawCertificate);

    Write-Host "Uploading K2 OAuth High Trust Certificate to $($Application.DisplayName)" -ForegroundColor Cyan;

    New-AzureADApplicationKeyCredential -ObjectId $Application.ObjectId -Type AsymmetricX509Cert -Usage Verify -Value $_value | Out-Null;
}

function Add-AppPermissions {

    param
    (
        $Application
    );

    $Graph = Get-AzureADServicePrincipal | Where { $_.DisplayName -eq "Microsoft Graph" };
    
    $GraphAccess = New-Object -TypeName "Microsoft.Open.AzureAD.Model.RequiredResourceAccess";
    
    $GraphAccess.ResourceAppId = $Graph.AppId;
    
    Write-Host 'Adding Graph Delegated Directory.Read.All access' -ForegroundColor Cyan;

    $DelegatedDirectoryReadAll = New-Object -TypeName "Microsoft.Open.AzureAD.Model.ResourceAccess" -ArgumentList "06da0dbc-49e2-44d2-8312-53f166ab848a","Scope";

    Write-Host 'Adding Graph Delegated User.Read access' -ForegroundColor Cyan;
    
    $DelegatedUserRead = New-Object -TypeName "Microsoft.Open.AzureAD.Model.ResourceAccess" -ArgumentList "e1fe6dd8-ba31-4d61-89e7-88639da4683d","Scope";

    Write-Host 'Adding Graph Application Directory.Read.All access' -ForegroundColor Cyan;

    $ApplicationDirectoryReadAll = New-Object -TypeName "Microsoft.Open.AzureAD.Model.ResourceAccess" -ArgumentList "7ab1d382-f21e-4acd-a863-ba3e13f7da61","Role";

    $GraphAccess.ResourceAccess = $DelegatedDirectoryReadAll, $DelegatedUserRead, $ApplicationDirectoryReadAll;

    $SharePoint = Get-AzureADServicePrincipal | Where { $_.DisplayName -eq "Office 365 SharePoint Online" };
    
    $SharePointAccess = New-Object -TypeName "Microsoft.Open.AzureAD.Model.RequiredResourceAccess";
    
    $SharePointAccess.ResourceAppId = $SharePoint.AppId;
    
    Write-Host 'Adding SharePoint Delegated AllSites.Full.Control access' -ForegroundColor Cyan;

    $DelegatedAllSitesFullControl = New-Object -TypeName "Microsoft.Open.AzureAD.Model.ResourceAccess" -ArgumentList "56680e0d-d2a3-4ae1-80d8-3c4f2100e3d0","Scope";

    Write-Host 'Adding SharePoint Delegated TermStore.ReadWrite.All access' -ForegroundColor Cyan;

    $DelegatedTermStoreReadWriteAll = New-Object -TypeName "Microsoft.Open.AzureAD.Model.ResourceAccess" -ArgumentList "59a198b5-0420-45a8-ae59-6da1cb640505","Scope";

    Write-Host 'Adding SharePoint Application Sites.Selected access' -ForegroundColor Cyan;

    $ApplicationSitesSelected = New-Object -TypeName "Microsoft.Open.AzureAD.Model.ResourceAccess" -ArgumentList "20d37865-089c-4dee-8c41-6967602d4ac8","Role";

    Write-Host 'Adding SharePoint Application TermStore.ReadWrite.All access' -ForegroundColor Cyan;

    $ApplicationTermStoreReadWriteAll = New-Object -TypeName "Microsoft.Open.AzureAD.Model.ResourceAccess" -ArgumentList "c8e3537c-ec53-43b9-bed3-b2bd3617ae97","Role";
    
    $SharePointAccess.ResourceAccess = $DelegatedAllSitesFullControl, $DelegatedTermStoreReadWriteAll, $ApplicationSitesSelected, $ApplicationTermStoreReadWriteAll;

    Set-AzureADApplication -ObjectId $Application.ObjectId -RequiredResourceAccess $GraphAccess,$SharePointAccess;
}

Load-AzModule;

Write-Host "Connecting to Azure..." -ForegroundColor Cyan;

Connect-AzureAD -TenantId $_tenantId | Out-Null;

$_app = Get-AzureADApplication | Where-Object { $_.DisplayName -eq $_appName };

if (!$_app) {

    Write-Host "Registering Azure Application - $_appName" -ForegroundColor Cyan;

    $_identifierUris = "$_url/Designer/", "$_url/Runtime/";
    
    $_replyUris = "$_url/Designer/", "$_url/Runtime/", "$_url/Identity/token/oauth/2";

    $_app = New-AzureADApplication -DisplayName $_appName -IdentifierUris $_identifierUris -ReplyUrls $_replyUris;
    
    $_secret = Add-ClientSecret -Application $_app;

    Add-HighTrustCertificate -Application $_app;

    New-AzureADServicePrincipal -AppId $_app.AppId | Out-Null;

    Add-AppPermissions -Application $_app;

    Write-Host;

    Write-Host "Client ID:"$_app.AppId -ForegroundColor Yellow;

    Write-Host "Client Secret:"$_secret -ForegroundColor Yellow;

} else {

    Write-Host "The custom application already exists in your tenant called $_appName";
}

Disconnect-AzureAD;

Read-Host 'Copy the Client Secret for later use, then press any key to exit';

Exit;